import os

from restfly.session import APISession

from pyzscaler.utils import obfuscate_api_key
from pyzscaler.version import version
from .config import ActivationAPI
from .locations import LocationsAPI
from .users import UserManagementAPI


class ZIA(APISession):
    """A Controller to access Endpoints in the Zscaler Internet Access (ZIA) API.

     The ZIA object stores the session token and simplifies access to CRUD options within the ZIA platform.

     Attributes:
         api_key (str): The ZIA API key generated from the ZIA console.
         username (str): The ZIA administrator username.
         password (str): The ZIA administrator password.

     """
    _vendor = 'Zscaler'
    _product = 'Zscaler Internet Access'
    _build = version
    _box = True
    _box_attrs = {
        'camel_killer_box': True
    }
    _env_base = 'ZIA'
    _env_cloud = 'zscaler'
    _url = 'https://zsapi.zscaler.net/api/v1'

    def __init__(self, **kw):
        self._api_key = kw.get('api_key',
                               os.getenv(f'{self._env_base}_API_KEY'))
        self._username = kw.get('username',
                                os.getenv(f'{self._env_base}_USERNAME'))
        self._password = kw.get('password',
                                os.getenv(f'{self._env_base}_PASSWORD'))
        self._env_cloud = kw.get('cloud',
                                 os.getenv(f'{self._env_base}_CLOUD'))
        self._url = f'https://zsapi.{self._env_cloud}.net/api/v1'
        super(ZIA, self).__init__(**kw)

    def _build_session(self, **kwargs) -> None:
        """Creates a ZIA API session using the /api/v1/authenticatedSession endpoint.

        :param kwargs:
        :return:
        """
        print(self._url)
        super(ZIA, self)._build_session(**kwargs)
        api_obf = obfuscate_api_key(self._api_key)

        payload = {
            'apiKey': api_obf['key'],
            'username': self._username,
            'password': self._password,
            'timestamp': api_obf['timestamp']
        }
        headers = {
            'Content-Type': 'application/json',
        }
        self.post('authenticatedSession', headers=headers, json=payload)

    @property
    def config(self):
        """
        The interface object for the :ref:`ZIA Activation interface <zia-config>`.

        """
        return ActivationAPI(self)

    @property
    def locations(self):
        """
        The interface object for the :ref:`ZIA Locations interface <zia-locations>`.

        """
        return LocationsAPI(self)

    @property
    def users(self):
        """
        The interface object for the :ref:`ZIA User Management interface <zia-users>`.

        """
        return UserManagementAPI(self)



from restfly.endpoint import APIEndpoint


class PolicySetsAPI(APIEndpoint):

    def list_policies(self, policy_type: str = None):
        """Returns the policy and rule sets for the given policy type.

        Args:
            policy_type (str):
                The type of policy to be returned. Accepted values are 'access', 'timeout' and 'client_forwarding'.

        Returns:
            :obj:`dict`: Resource record of the specified policy.

        Examples:
            Request the specified Policy.

            >>> pprint(zpa.policies.list_policies('access'))

        """
        _policy_url = None

        if policy_type == 'access':
            _policy_url = 'global'
        elif policy_type == 'timeout':
            _policy_url = 'reauth'
        elif policy_type == 'client_forwarding':
            _policy_url = 'bypass'

        return self._get(f'policySet/{_policy_url}')

    def get_rule(self, policy_id: str, rule_id: str):
        """Returns the specified policy rule.

        Args:
            policy_id (str):
                The unique identifier for the policy.
            rule_id (str):
                The unique identifier for the policy rule.

        Returns:
            :obj:`dict`: The resource record for the requested rule.

        Examples:
            >>> policy_rule = zpa.policies.get_rule(policy_id='234234234344',
            ...    rule_id='23456546546456')

        """

        return self._get(f'policySet/{policy_id}/rule/{rule_id}')

    def list_rules(self, policy_type: str):
        """Returns policy rules for a given policy type.

        Args:
            policy_type (str):
                The policy type. Accepted values are:
                GLOBAL_POLICY - returns Access Policies
                REAUTH_POLICY - returns Timeout Policies
                BYPASS_POLICY - returns Client Forwarding Policies

        Returns:
            :obj:`list`: A list of all policy rules that match the requested type.

        Examples:
            >>> for policy in zpa.policies.list_type('type')
            ...    pprint(policy)

        """

        return self._get(f'policySet/rules/policyType/{policy_type}').list

    def delete_rule(self, policy_type: str, rule_id: str):
        """Deletes the specified policy rule.

        Args:
            policy_type (str):
                The type of policy the rule belongs to. Accepted values are 'access', 'timeout' and 'client_forwarding'.
            rule_id (str):
                The unique identifier for the policy rule.

        Returns:
            :obj:`str`: The response code for the operation.

        Examples:
            >>> zpa.policies.delete_rule(policy_id='2342567567355535',
            ...    rule_id='23322646655778')

        """

        # Get policy id for specified policy type
        _policy_id = self.list_policies(policy_type).id

        return self._delete(f'policySet/{_policy_id}/rule/{rule_id}')

    def add_access_rule(self, name: str, action: str = None, **kwargs):
        """Add a new Access Policy rule.

        See the `ZPA Access Policy API reference <https://help.zscaler.com/zpa/access-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            action (str, optional):
                The action for the policy. Accepted values are:
                'ALLOW' (default) or 'DENY'.
            **kwargs:
                Optional parameters.

        Keyword Args:
            conditions (list:
                List of conditions.
            customMsg (str):
                A custom message.
            description (str):
                A description for the rule.

        Returns:
            :obj:`dict`: The resource record of the newly created access policy rule.

        """

        # Initialise the payload
        payload = {
            'name': name,
        }

        # Get the policy id for the provided policy type and configure payload accordingly.
        _policy_id = self.list_policies('access').id
        payload['action'] = kwargs.get('action', 'ALLOW')

        for key, value in kwargs.items():
            payload[key] = value

        return self._post(f'policySet/{_policy_id}/rule', json=payload)

    def add_timeout_rule(self, name: str, **kwargs):
        """Add a new Timeout Policy rule.

        See the `ZPA Timeout Policy API reference <https://help.zscaler.com/zpa/timeout-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            **kwargs:
                Optional parameters.

        Keyword Args:
            conditions (list):
                List of conditions.
            customMsg (str):
                A custom message.
            description (str):
                A description for the rule.
            re_auth_idle_timeout (int):
                The re-authentication idle timeout value in seconds.
            re_auth_timeout (int):
                The re-authentication timeout value in seconds.

        Returns:
            :obj:`dict`: The resource record of the newly created Timeout Policy rule.

        """

        # Initialise the payload
        payload = {
            'name': name,
        }

        # Get the policy id for the provided policy type and configure payload accordingly.
        _policy_id = self.list_policies('timeout').id
        payload['action'] = 'RE_AUTH'
        payload['reauthTimeout'] = kwargs.get('re_auth_timeout', 172800)  # Default value in ZPA UI
        payload['reauthIdleTimeout'] = kwargs.get('re_auth_idle_timeout', 600)  # Default value in ZPA UI

        for key, value in kwargs.items():
            payload[key] = value

        return self._post(f'policySet/{_policy_id}/rule', json=payload)

    def add_client_forwarding_rule(self, name: str, action: str = None, **kwargs):
        """Add a new Client Forwarding Policy rule.

        See the `ZPA Client Forwarding Policy API reference <https://help.zscaler.com/zpa/client-forwarding-policy-use-cases>`_
        for further detail on optional keyword parameter structures.

        Args:
            name (str):
                The name of the new rule.
            action (str, optional):
                The action for the policy. Accepted values are:
                'INTERCEPT' (default), 'INTERCEPT_ACCESSIBLE' and 'BYPASS'.
            **kwargs:
                Optional parameters.

        Keyword Args:
            conditions (list:
                List of conditions.
            customMsg (str):
                A custom message.
            description (str):
                A description for the rule.

        Returns:
            :obj:`dict`: The resource record of the newly created Client Forwarding Policy rule.

        """

        # Initialise the payload
        payload = {
            'name': name,
        }

        # Get the policy id for the provided policy type and configure payload accordingly.
        _policy_id = self.list_policies('client_forwarding').id
        payload['action'] = kwargs.get('action', 'INTERCEPT')

        for key, value in kwargs.items():
            payload[key] = value

        return self._post(f'policySet/{_policy_id}/rule', json=payload)

    def update_rule(self, policy_type: str, rule_id: str = None, name: str = None, action: str = None, **kwargs):
        """Update an existing policy rule

        Passing only the policy_type and rule_id will result in a successful operation with no changes to the rule
        record. All other supplied params will result in a change.

        Args:
            policy_type:
                The policy type. Accepted values are 'access', 'timeout' and 'client_forwarding'
            rule_id:
                The unique identifier for the rule to be updated.
            name:
                The name of the rule. Defaults to existing name if omitted.
            action:
                The action of the rule. Defaults to existing action if omitted.
            **kwargs:

        Returns:
            :obj:`str`: The response code for the operation.

        Examples:

            Updates the name only for an Access Policy rule:

            >>> zpa.policies.update_rule('access', '72057615512764594', name='new_rule_name')

            Updates the action only for a Client Forwarding Policy rule:

            >>> zpa.policies.update_rule('client_forwarding', '72057615512764595', action='BYPASS')

        """

        # Get the policy id for the supplied policy_type
        _policy_id = self.list_policies(policy_type).id

        # Name required so use existing name from rule if not specified
        if not name:
            name = self.get_rule(_policy_id, rule_id).name

        # Action required so use existing action from rule if not specified
        if not action:
            action = self.get_rule(_policy_id, rule_id).action

        # Initialise payload
        payload = {
            'name': name,
            'action': action
        }

        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._put(f'policySet/{_policy_id}/rule/{rule_id}', json=payload, box=False).status_code



from restfly.utils import dict_merge, url_validator
import time


def snake_to_camel(name):
    name = name[0].lower() + name.title()[1:].replace("_", "")
    return name


def obfuscate_api_key(seed):
    now = int(time.time() * 1000)
    n = str(now)[-6:]
    r = str(int(n) >> 1).zfill(6)
    key = ""
    for i in range(0, len(str(n)), 1):
        key += seed[int(str(n)[i])]
    for j in range(0, len(str(r)), 1):
        key += seed[int(str(r)[j]) + 2]

    return {'timestamp': now, 'key': key}


from restfly.endpoint import APIEndpoint


class ActivationAPI(APIEndpoint):

    def status(self):
        """
        Returns the activation status for a configuration change.

        Returns:
            :obj:`str`
                Configuration status.

        Examples:
            >>> config_status = zia.config.status()

        """
        return self._get('status').status

    def activate(self):
        """
        Activates configuration changes.

        Returns:
            :obj:`str`
                Configuration status.

        Examples:
            >>> config_activate = zia.config.activate()

        """
        return self._post('status/activate').status



from restfly.endpoint import APIEndpoint
from box import BoxList
from pyzscaler.utils import snake_to_camel


class LocationsAPI(APIEndpoint):

    def list_locations(self):
        """
        Returns a list of configured locations.

        Returns:
            :obj:`list`: List of configured locations.

        Examples:
            >>> locations = zia.locations.list_locations()

        """
        return self._get('locations', box=BoxList)

    def add_location(self, name: str, **kwargs):
        """
        Adds a new location.

        Args:
            name (str):
                Location name.

        Keyword Args:
            ip_addresses (list):
                    For locations: IP addresses of the egress points that are provisioned in the Zscaler Cloud.
                    Each entry is a single IP address (e.g., 238.10.33.9).

                    For sub-locations: Egress, internal, or GRE tunnel IP addresses. Each entry is either a single
                    IP address, CIDR (e.g., 10.10.33.0/24), or range (e.g., 10.10.33.1-10.10.33.10)).
            ports (:obj:`list` of :obj:`str`):
                List of whitelisted Proxy ports for the location.
            vpn_credentials (dict):
                VPN credentials for the location.

        Returns:
            :obj:`dict`: The newly created location resource record

        Examples:
            Add a new location with an IP address.

            >>> zia.locations.add_location(name='new_location',
            ...    ip_addresses=['203.0.113.10'])

        """
        payload = {
            'name': name,
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post('locations', json=payload)

    def get_location(self, location_id: str):
        """
        Returns information for the specified location.

        Args:
            location_id (str):
                The unique identifier for the location.

        Returns:
            :obj:`dict`: The requested location resource record.

        Examples:
            >>> location = zia.locations.get_location('97456691')
        """
        return self._get(f'locations/{location_id}')

    def list_sub_locations(self, location_id: str):
        """
        Returns sub-location information for the specified location ID.

        Args:
            location_id (str):
                The unique identifier for the parent location.

        Returns:
            :obj:`list`: A list of sub-locations configured for the parent location.

        Examples:
            >>> for sub_location in zia.locations.list_sub_locations('97456691'):
            ...    pprint(sub_location)

        """
        return self._get(f'locations/{location_id}/sublocations', box=BoxList)

    def list_locations_lite(self):
        """
        Returns only the name and ID of all configured locations.

        Returns:
            :obj:`list`: A list of configured locations.

        Examples:
            >>> for location in zia.locations.list_locations_lite():
            ...    pprint(location)

        """
        return self._get('locations/lite', box=BoxList)

    def update_location(self, location_id: str, **kwargs):
        """Update the specified location.

        Note: Changes are not additive and will replace existing values.

        Args:
            location_id (str):
                The unique identifier for the location you are updating.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            ip_addresses (:obj:`list` of :obj:`str`):
                List of updated ip addresses.
            ports (:obj:`list` of :obj:`str`):
                List of whitelisted Proxy ports for the location.
            vpn_credentials (dict):
                VPN credentials for the location.

        Returns:
            :obj:`dict`: The updated resource record.

        Examples:
            Update the name of a location:

            >>> zia.locations.update('97456691',
            ...    name='updated_location_name')

            Upodate the IP address of a location:

            >>> zia.locations.update('97456691',
            ...    ip_addresses=['203.0.113.20'])

        """
        location_record = self.get_location(location_id)

        payload = {}

        # Check if required params are provided, if not, add to payload from existing record.
        if not kwargs.get('ip_addresses') and 'ip_addresses' in location_record:
            payload['ipAddresses'] = location_record['ip_addresses']

        if not kwargs.get('ports') and 'ports' in location_record:
            payload['ports'] = location_record['ports']

        if not kwargs.get('vpn_credentials') and 'vpnCredentials' in location_record:
            payload['vpnCredentials'] = location_record['vpn_credentials']

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._put(f'locations/{location_id}', json=payload)

    def delete_location(self, location_id: str):
        """
        Deletes the location or sub-location for the specified ID

        Args:
            location_id (str):
                The unique identifier for the location or sub-location.

        Returns:
            :obj:`str`: Response code for the operation.

        Examples:
            >>> zia.locations.delete_location('97456691')

        """
        return self._delete(f'locations/{location_id}', box=False).status_code



from restfly.endpoint import APIEndpoint


class AppServersAPI(APIEndpoint):

    def add_server(self, name: str = None, address: str = None, enabled: bool = False, **kwargs):
        """
        Add a new application server.

        Args:
            name (str, required):
                The name of the server.
            address (str, required):
                The IP address of the server.
            enabled (bool, required):
                 Should the server be enabled. Defaults to False.
            **kwargs:

        Keyword Args:
            description (str, optional):
                A description for the server.
            appServerGroupIds (:obj:`list` of :obj:`str`), optional):
                Unique identifiers for the server groups the server belongs to.
            configSpace (str, optional):
                The configuration space for the server. Defaults to DEFAULT.

        Returns:
            :obj:`dict`: The resource record for the newly created server.

        Examples:
            Create a server with the minimum required parameters:

            >>> zpa.servers.add_server(
            ...   name='myserver.example',
            ...   address='192.0.2.10',
            ...   enabled=True)

        """
        payload = {
            "name": name,
            "address": address,
            "enabled": enabled,
            "description": kwargs.get('description', ''),
            "appServerGroupIds": kwargs.get('server_group_ids', []),
            "configSpace": kwargs.get('config_space', 'DEFAULT')
        }
        return self._post('server', json=payload)

    def list_servers(self):
        """
        Returns all configured servers.

        Returns:
            :obj:`list`: List of all configured servers.

        Examples:
            >>> servers = zpa.servers.list_servers()
        """
        return self._get('server').list

    def get_server(self, server_id: str):
        """
        Get information for the specified server id.

        Args:
            server_id (str):
                The unique identifier for the server.

        Returns:
            :obj:`dict`: The resource record for the server.

        Examples:
            >>> server = zpa.servers.get_server('12')

        """
        return self._get(f'server/{server_id}')

    def delete_server(self, server_id: str):
        """
        Delete the specified server.

        Args:
            server_id (str):
                The unique identifier for the server to be deleted.

        Returns:
            :obj:`str`: The response code for the operation.

        Examples:
            >>> zpa.servers.delete_server('32')

        """
        return self._delete(f'server/{server_id}')

    def update_server(self, server_id: str, **kwargs):
        """Updates the specified server

        Args:
            server_id (str):
                The unique identifier for the server being updated.
            **kwargs:

        Keyword Args:
            name (str, optional):
                The name of the server.
            address (str, optional):
                The IP address of the server.
            enabled (bool, optional):
                 Should the server be enabled.
            description (str, optional):
                A description for the server.
            appServerGroupIds (list(str), optional):
                Unique identifiers for the server groups the server belongs to.
            configSpace (str, optional):
                The configuration space for the server.

        Returns:
            :obj:`dict`: The resource record for the updated server.

        Examples:
            Update the name of a server:

            >>> zpa.servers.update_server(
            ...   '23483234823484',
            ...   name='newname.example')

            Update the address and enable a server:

            >>> zpa.servers.update_server(
            ...    '23483234823484',
            ...    address='192.0.2.20',
            ...    enabled=True)

        """
        payload = {
            'id': server_id
        }

        for key, value in kwargs.items():
            payload[key] = value

        return self._put(f'server/{server_id}', json=payload)



from restfly.endpoint import APIEndpoint


class ServerGroupsAPI(APIEndpoint):

    def list_groups(self):
        """
        Returns a list of all configured server groups.

        Returns:
            :obj:`list`: A list of all configured server groups.

        Examples:
            >>> for server_group in zpa.server_groups.list_groups():
            ...    pprint(server_group)

        """
        return self._get('serverGroup').list

    def get_group(self, group_id: str):
        """
        Provides information on the specified server group.

        Args:
            group_id (str):
                The unique identifier for the server group.

        Returns:
            :obj:`dict`: The resource record for the server group.

        Examples:
            >>> pprint(zpa.server_groups.get_group('2342342342344433'))

        """

        return self._get(f'serverGroup/{group_id}')

    def delete_group(self, group_id: str):
        """
        Deletes the specified server group.

        Args:
            group_id (str):
                The unique identifier for the server group to be deleted.

        Returns:
            :obj:`str`: The response code for the operation.

        Examples:
            >>> zpa.server_groups.delete_group('2342342342343')

        """
        return self._delete(f'serverGroup/{group_id}')

    def add_group(self, name: str, app_connector_ids: list, **kwargs):
        """Adds a server group.

        Args:
            name (str):
                The name for the server group.
            app_connector_ids (:obj:`list` of :obj:`str`):
                A list of application connector IDs that will be attached to the server group.
            **kwargs:
                Optional params.

        Keyword Args:
            applications (list):
            configSpace (str):
            description (str):
            enabled (bool):
            ipAnchored (bool):
            dynamicDiscovery (bool):
            servers (list):

        Returns:
            :obj:`dict`: The resource record for the newly created server group.

        Examples:
            Create a server group with the minimum params:

            >>> zpa.server_groups.add_group('new_server_group'
            ...    app_connector_ids['23423423432444'])

            Create a server group and define a new server on the fly:

            >>> zpa.server_groups.add_group('new_server_group',
            ...    app_connector_ids=['23423423432444'],
            ...    enabled=True,
            ...    servers=[{
            ...      'name': 'new_server',
            ...      'address': '10.0.0.30',
            ...      'enabled': True}])

        """
        # Initialise payload
        payload = {
            'name': name,
            'appConnectorGroups': []
        }
        # Iterate through provided app connector group IDs and add to payload
        for app_connector_id in app_connector_ids:
            app_connector_group = {
                'id': app_connector_id
            }
            payload['appConnectorGroups'].append(app_connector_group)
        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._post('serverGroup', json=payload)

    def update_group(self, group_id: str, dynamic_discovery: bool = None, servers: list = None, **kwargs):
        """
        Updates a server group.

        .. Note:: The ZPA server group API requires the dynamic_discovery and servers params to be provided for all
            updates.

        Args:
            group_id (str, required):
                The unique identifier for the server group.
            dynamic_discovery (bool, required):
                Should Dynamic Discovery be enabled.
            servers (:obj:`list` of :obj:`dict`):
                The server objects for the server group.
            **kwargs:

        Returns:
            :obj:`dict`: The resource record for the updated server group.

        """

        payload = {
            'id': group_id,
        }
        if dynamic_discovery is None:
            payload['dynamicDiscovery'] = dynamic_discovery

        if servers is not None:
            payload['servers'] = servers

        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._put(f'serverGroup/{group_id}', json=payload, box=False)



from restfly.endpoint import APIEndpoint


class AppSegmentsAPI(APIEndpoint):

    def list_segments(self):
        """Retrieve all configured application segments.

        Returns:
            :obj:`list`: List of application segments.

        Examples:
            >>> app_segments = zpa.app_segments.list_segments()

        """
        return self._get('application').list

    def details(self, segment_id: str):
        """Get information for an application segment.

        Args:
            segment_id (str):
                The unique identifier for the application segment.

        Returns:
            :obj:`dict`: The application segment resource record.

        Examples:
            >>> app_segment = zpa.app_segments.details('234324234324')

        """
        return self._get(f'application/{segment_id}')

    def delete_segment(self, segment_id: str):
        """Delete an application segment.

        Args:
            segment_id (str):
                The unique identifier for the application segment.

        Returns:
            :obj:`str`: The operation response code.

        Examples:
            >>> zpa.app_segments.delete('234324234324')

        """
        return self._delete(f'application/{segment_id}')

    def add_segment(self, name: str, domain_names: list, segment_group_id: str, server_groups: list,
                    tcp_ports: str = None, udp_ports: str = None, **kwargs):
        """Create an application segment.

        Args:
            segment_group_id (str):
                The unique identifer for the segment group this application segment belongs to.
            udp_ports (:obj:`list` of :obj:`str`):
                List of udp port range pairs, e.g. ['35000', '35000'] for port 35000.
            tcp_ports (:obj:`list` of :obj:`str`):
                List of tcp port range pairs, e.g. ['22', '22'] for port 22-22, ['80', '100'] for 80-100.
            domain_names (:obj:`list` of :obj:`str`):
                List of domain names or IP addresses for the application segment.
            name (str):
                The name of the application segment.
            server_groups (:obj:`list` of :obj:`str`):
                The list of server group IDs that belong to this application segment.
            **kwargs:
                Optional params.

        Keyword Args:
            bypassType (str):
            clientlessApps (:obj:`list`):
            configSpace (str):
            defaultIdleTimeout (str):
            defaultMaxAge (str):
            description (str):
            doubleEncrypt (bool):
            enabled (bool):
            healthCheckType (str):
            healthReporting (str):
            ipAnchored (bool):
            isCnameEnabled (bool):
            passiveHealthEnabled (bool):

        Returns:
            :obj:`dict`: The newly created application segment resource record.

        Examples:
            Add a new application segment for example.com, ports 8080-8085.

            >>> zpa.app_segments.add_segment('new_app_segment',
            ...    domain_names=['example.com'],
            ...    segment_group='232356567677776',
            ...    tcp_ports=['8080', '8085'],
            ...    server_groups=['234234234234324', '23121115151515'])

        """

        # Replace placeholders
        if udp_ports is None:
            udp_ports = []
        if tcp_ports is None:
            tcp_ports = []

        # Initialise payload
        payload = {
            'name': name,
            'domainNames': domain_names,
            'tcpPortRanges': tcp_ports,
            'udpPortRanges': udp_ports,
            'segmentGroupId': segment_group_id,
            'serverGroups': []
        }

        # Iterate through provided server group IDs and add to payload
        for server_group_id in server_groups:
            server_group = {
                'id': server_group_id
            }
            payload['serverGroups'].append(server_group)

        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._post('application', json=payload)

    def update_segment(self, segment_id: str, domain_names: list, **kwargs):
        """Update an application segment.

        .. note:: The ZPA API requires domain names to be passed for all updates, even if they don't require modification.

        Args:
            segment_id (str):
                The unique identifier for the application segment.
            domain_names (:obj:`list` of :obj:`str`):
                List of domain names or IP addresses for the application segment.
            **kwargs:
                Optional params.

        Keyword Args:
            bypassType (str):
            clientlessApps (:obj:`list`):
            configSpace (str):
            defaultIdleTimeout (str):
            defaultMaxAge (str):
            description (str):
            doubleEncrypt (bool):
            enabled (bool):
            healthCheckType (str):
            healthReporting (str):
            ipAnchored (bool):
            isCnameEnabled (bool):
            name (str):
            passiveHealthEnabled (bool):
            segmentGroupId (str):
            serverGroups (:obj:`list`):
            tcpPortRanges (:obj:`list`):
            udpPortRanges (:obj:`list`):

        Returns:
            :obj:`dict`: The updated application segment resource record.

        Examples:
            Rename the application segment for example.com.

            >>> zpa.app_segments.update('2234234123234523',
            ...    name='new_app_name',
            ...    domain_names=['example.com'])

        """

        # Initialise payload
        payload = {
            'domainNames': domain_names
        }

        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._put(f'application/{segment_id}', json=payload)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyzscaler', 'pyzscaler.zia', 'pyzscaler.zpa']

package_data = \
{'': ['*']}

install_requires = \
['python-box>=5.3.0,<6.0.0', 'restfly>=1.4.0,<2.0.0']

setup_kwargs = {
    'name': 'pyzscaler',
    'version': '0.2.2',
    'description': 'A python SDK for Zscaler.',
    'long_description': 'pyZscaler is an unofficial SDK for interacting with Zscaler APIs\n=====================================================================\npyZscaler aims to provide a uniform and easy-to-use interface for each of the Zscaler product APIs.\n\n\n.. attention:: This SDK is not affiliated with, nor supported by Zscaler in any way.\n\n   :strong:`Caveats`\n\n   - Not all features may be implemented.\n   - Implemented features may be buggy or incorrect.\n   - Bugs will be fixed in my own time.\n\nOverview\n==========\nWith each Zscaler product having its own developer documentation and authentication methods, this SDK should simplify\nyour ability to develop software that uses the Zscaler API.\n\nThis SDK leverages the very awesome `RESTfly framework <https://restfly.readthedocs.io/en/latest/index.html>`_ developed by Steve McGrath, which simplifies the development of\nbuilding libraries to interact with RESTful APIs. A big thank you to Steve.\n\nFeatures\n----------\n- Simplified authentication with Zscaler APIs.\n- Uniform interaction with all Zscaler APIs.\n- Uses `python-box <https://github.com/cdgriffith/Box/wiki>`_ to add dot notation access to json data structures.\n- Zscaler API output automatically converted from CamelCase to Snake Case.\n\nProducts\n---------\n- Zscaler Private Access (ZPA)\n- Zscaler Internet Access (ZIA)\n- Cloud Security Posture Management (CSPM) - (work in progress)\n\nInstallation\n==============\n\nThe most recent version can be installed from pypi as per below.\n\n.. code-block:: console\n\n    $ pip install pyzscaler\n\nUsage\n========\nBefore you can interact with any of the Zscaler APIs, you may need to generate API keys or retrieve tenancy information\nfor each product that you are interfacing with. Once you have the requirements and you have installed pyZscaler,\nyou\'re ready to go.\n\nGetting started with ZIA\n--------------------------\nFor ZIA, you will need to provide params when instantiating the class or set the environment variables as per the\ntable below:\n\n.. list-table:: ZIA Requirements\n   :header-rows: 1\n\n   * - Param\n     - ENV\n     - Description\n   * - cloud\n     - ``ZIA_CLOUD``\n     - The cloud that your ZIA tenant is provisioned on.\n   * - api_key\n     - ``ZIA_API_KEY``\n     - The API key generated from your ZIA console.\n   * - username\n     - ``ZIA_USERNAME``\n     - The username of your administrator user that will be used for API calls.\n   * - password\n     - ``ZIA_PASSWORD``\n     - The password for the administrator user.\n\n`How to generate the API_KEY <https://help.zscaler.com/zia/api-getting-started#RetrieveAPIKey>`_\n\nHow to determine the ZIA CLOUD\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nYou can find the name of the cloud in the URL that admins use to log into the Zscaler service. E.g.\nif an organisation logs into admin.zscaler.net, then that organization\'s cloud name is zscaler.net. You don\'t\nneed to supply the .net suffix with pyZscaler, so the ``CLOUD`` arg would simply be ``zscaler``.\n\n.. list-table:: ZIA Cloud List\n   :header-rows: 1\n\n   * - URL\n     - CLOUD\n   * - admin.zscaler.net\n     - zscaler\n   * - admin.zscalerone.net\n     - zscalerone\n   * - admin.zscalertwo.net\n     - zscalertwo\n   * - admin.zscalerthree.net\n     - zscalerthree\n   * - admin.zscloud.net\n     - zscloud\n   * - admin.zscalerbeta.net\n     - zscalerbeta\n   * - admin.zscalergov.net\n     - zscalergov\n\n\n.. code-block:: python\n\n    from pyzscaler import ZIA\n    zia = ZIA(api_key=\'API_KEY\', cloud=\'CLOUD\', username=\'USERNAME\', password=\'PASSWORD\')\n    for user in zia.users.list():\n        pprint(user)\n\nGetting started with ZPA\n--------------------------\nFor ZPA, you will need to provide params when instantiating the class or set the environment variables as per the\ntable below:\n\n.. list-table:: ZPA Requirements\n   :header-rows: 1\n\n   * - Param\n     - ENV\n     - Description\n   * - client_id\n     - ``ZPA_CLIENT_ID``\n     - The client ID that is associated with the client secret.\n   * - client_secret\n     - ``ZPA_CLIENT_SECRET``\n     - The client secret that was generated for the client ID.\n   * - customer_id\n     - ``ZPA_CUSTOMER_ID``\n     - The customer ID for the ZPA tenancy.\n\n- `How to generate the CLIENT_ID, CLIENT_SECRET and find the CUSTOMER_ID <https://help.zscaler.com/zpa/about-api-keys>`_\n\n\n.. code-block:: python\n\n    from pyzscaler import ZPA\n    zpa = ZPA(client_id=\'CLIENT_ID\', client_secret=\'CLIENT_SECRET\', customer_id=\'CUSTOMER_ID\')\n    for app_segment in zpa.app_segments.list():\n        pprint(app_segment)\n\n\nLicense\n=========\nMIT License\n\nCopyright (c) 2021 Mitch Kelly\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.',
    'author': 'Mitch Kelly',
    'author_email': 'me@mkelly.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://mitchos.github.io/pyZscaler/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from restfly.endpoint import APIEndpoint


class SegmentGroupsAPI(APIEndpoint):

    def list_groups(self):
        """
        Returns a list of all configured segment groups.

        Returns:
            :obj:`list`: A list of all configured segment groups.

        Examples:
            >>> for segment_group in zpa.segment_groups.list_groups():
            ...    pprint(segment_group)

        """
        return self._get('segmentGroup').list

    def get_group(self, group_id: str):
        """
        Returns information on the specified segment group.

        Args:
            group_id (str):
                The unique identifier for the segment group.

        Returns:
            :obj:`dict`: The resource record for the segment group.

        Examples:
            >>> pprint(zpa.segment_groups.get_group('2342342342344433'))

        """

        return self._get(f'segmentGroup/{group_id}')

    def delete_group(self, group_id: str):
        """
        Deletes the specified segment group.

        Args:
            group_id (str):
                The unique identifier for the segment group to be deleted.

        Returns:
            :obj:`str`: The response code for the operation.

        Examples:
            >>> zpa.segment_groups.delete_group('2342342342343')

        """
        return self._delete(f'segmentGroup/{group_id}')

    def add_group(self, name: str, enabled=False, **kwargs):
        """
        Adds a new segment group.

        Args:
            name (str):
                The name of the new segment group.
            enabled (bool):
                Enable the segment group. Defaults to False.
            **kwargs:

        Keyword Args:
            applications (:obj:`list` of :obj:`dict`):
                Define an application object for the segment group.
            configSpace (str):
                The config space for the segment group. Can either be DEFAULT or SIEM.
            description (str):
                A description for the segment group.
            policyMigrated (bool):

        Returns:
            :obj:`dict`: The resource record for the newly created segment group.

        Examples:
            Creating a segment group with the minimum required parameters:

            >>> zpa.segment_groups.add_group('new_segment_group',
            ...    True)

        """
        payload = {
            'name': name,
            'enabled': enabled,
        }
        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._post('segmentGroup', json=payload)

    def update_group(self, group_id: str, **kwargs):
        """
        Updates an existing segment group.

        Args:
            group_id (str):
                The unique identifier for the segment group to be updated.
            **kwargs:

        Keyword Args:
            name (str):
                The name of the new segment group.
            enabled (bool):
                Enable the segment group.
            applications (:obj:`list` of :obj:`dict`):
                Define an application object for the segment group.
            configSpace (str):
                The config space for the segment group. Can either be DEFAULT or SIEM.
            description (str):
                A description for the segment group.
            policyMigrated (bool):

        Returns:
            :obj:`dict`: The resource record for the updated segment group.

        Examples:
            Updating the name of a segment group:

            >>> zpa.segment_groups.update_group('23234234324234',
            ...    name='updated_name')

        """
        payload = {
            'id': group_id,
        }
        # Add optional params to payload
        for key, value in kwargs.items():
            payload[key] = value

        return self._put(f'segmentGroup/{group_id}', json=payload)
